/************************************************************************
	delay.c

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>

#include "delay.h"
#include "hardware.h"

void delayMilliSeconds(int milliSeconds)
{
	// This delay routine uses a long int (32 bit) counter which is 
	// incremented on every timer0 interrupt.
	//
	// Timer0 is called every 256 timer ticks and has a prescaler of 1:2
	// Fosc is 48,000,000, Fosc/4 is 12,000,000 ticks per second and the
	// 1:2 prescaler leave us at 6,000,000 ticks per second.
	//
	// This means that the delay counter is incremented 23529.5 times a second
	// (since the interrupt is called every 256 ticks)
	//
	// There are 1000 milliseconds in a second, so one millisecond is
	// 23529.5 / 1000 = 23.5295 increments of the delay timer

	long delayComparison;

	hardware.delayCounter = 0;

	// Starting with 23 I kept reducing the figure (to allow for the overhead)
	// until 10 milliseconds delay was accurate (the delay was measured with
	// an oscilloscope)
	delayComparison = 16 * milliSeconds;

	while (hardware.delayCounter < delayComparison);
}