/************************************************************************
	hardware.c

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#include <htc.h>
#include "hardware.h"

// Initialise the hardware and game states
void initialise(void)
{
	unsigned char loop;

	// Initialise hardware state
	hardware.lastButton.state = OFF;
	hardware.lastButton.debounceCount = 0;

	hardware.longestButton.state = OFF;
	hardware.longestButton.debounceCount = 0;

	hardware.startButton.state = OFF;
	hardware.startButton.debounceCount = 0;

	for (loop = 0; loop < 4; loop++)
	{
		hardware.colourButton[loop].state = OFF;
		hardware.colourButton[loop].debounceCount = 0;

		hardware.light[loop].state = OFF;
		hardware.light[loop].brightness = 0;
		hardware.light[loop].dimSpeed = 100;

		game.colourActive[loop] = TRUE;
	}

	hardware.gameSwitch.position = 1;
	hardware.skillSwitch.position = 1;

	hardware.sound.state = OFF;
	hardware.sound.pwmState = OFF;

	// This counter is used by the PWM light controls, it runs from 
	// 0 to 100 and is used as a comparison to the brightness setting
	// to determine if the light should be on or off for the current
	// interrupt call.
	hardware.pwmCounter = 0;

	// This counter is used to slow down the dimming (since the 
	// brightness is only 0 to 100 even dimming 1 unit per
	// interrupt would be too fast to see.  This counter ensures
	// dimming is only performed every few hundred calls to the 
	// interrupt.
	hardware.dimCounter = 0;

	// This counter is used to control the speed of the the button polling
	hardware.buttonCounter = 0;

	// This counter is used to time delays
	hardware.delayCounter = 0;

	// Initialise game state
	game.gameNumber = 1;
	game.skillLevel = 1;
	game.gameActive = OFF;
	game.currentSequencePointer = -1;
	game.playerSequencePointer = -1;
	game.longestSequenceLength = -1;
	game.randomNumber = 0;
	game.restartFlag = FALSE;

	// Initialise the buttons ready for the interrupt to read
	START_SKILL = ON;
	LAST_LONGEST_GAME = OFF;
	COLOUR_BUTTONS = OFF;
}